// Ref : http://www.esp8266.com/viewtopic.php?f=28&t=2295&p=13730#p13730

#include <ESP8266WiFi.h>
#include <WiFiUdp.h>

// H-bridge pin connections
#define LM0 D2
#define LM1 D1
#define RM0 D3
#define RM1 D4
// #define ENL D7
// #define ENR D6

// Defines used by the motor() function
#define STP 0
#define FOR 1
#define REV 2

// Delay (after last packet received) after which the car is stopped
#define STOPDELAY 220

unsigned long lastRxTime;// Stores time for last packet received
char carStopped = 0; // 0: car is moving, other: car is stopped

WiFiUDP port;
char packetBuffer[255];
unsigned int localPort = 42042;
const char WiFiAPPSK[] = "stuntman";

void setup() {
  Serial.begin(115200); // Init serial port
  setupWiFi(); // Init Wifi in Acces Point mode
  port.begin(localPort); // Start listening on UDP port

  // Set H-bridge pins to output mode
  pinMode(LM0, OUTPUT);
  pinMode(LM1, OUTPUT);
  pinMode(RM0, OUTPUT);
  pinMode(RM1, OUTPUT);
  // pinMode(ENL, OUTPUT);
  // pinMode(ENR, OUTPUT);

  // Set H-bridge to neutral (stop the motors)
  digitalWrite(LM0, LOW);
  digitalWrite(LM1, LOW);
  digitalWrite(RM0, LOW);
  digitalWrite(RM1, LOW);
  // digitalWrite(ENL, HIGH);
  // digitalWrite(ENR, HIGH);
  
}

void loop() {
  int packetSize = port.parsePacket();
  // Serial.println(packetSize);
  
  // If a packet has been received
  if (packetSize) {
    int len = port.read(packetBuffer, 255); // get packet length
    if (len > 0) packetBuffer[len] = 0; // if packet contains any data
    // Serial.println(packetBuffer);
    decodePacket(packetBuffer, len); // move the vehicle according to packet
    lastRxTime = millis(); // Store last received packet time
    carStopped = 0; // Car is moving
  }
  delay(10); 

  if((millis() - lastRxTime > STOPDELAY) && !carStopped) {
  	motor(STP, STP); // Stop car if out of range
  	carStopped = 1;  // Send the stop command only once
  }
}

// Packet decoding function: determines stunt vehicle direction
void decodePacket(char* packet, int len) {
  if(packet[0]=='3' && packet[1]=='3') motor(STP,STP);
  else if(packet[1]=='1' && packet[0]=='3') motor(FOR,FOR);
  else if(packet[1]=='2' && packet[0]=='3') motor(REV,REV);

  else if(packet[1]=='3' && packet[0]=='1') motor(REV,FOR);
  else if(packet[1]=='3' && packet[0]=='2') motor(FOR,REV);

  else if(packet[1]=='1' && packet[0]=='1') motor(STP,FOR);
  else if(packet[1]=='1' && packet[0]=='2') motor(FOR,STP);
  else if(packet[1]=='2' && packet[0]=='1') motor(STP,REV);
  else if(packet[1]=='2' && packet[0]=='2') motor(REV,STP);

  else if(packet[1]=='4' && packet[0]=='4') stunt(1);
  else if(packet[1]=='5' && packet[0]=='5') stunt(2);
  else if(packet[1]=='6' && packet[0]=='6') stunt(3);
}

void motor(int leftDir, int rightDir) {
  switch(leftDir) {
    case STP: //stop
      digitalWrite(LM0, LOW);
      digitalWrite(LM1, LOW);
      // Serial.println("left stop");
      break;
    case FOR: //forward
      digitalWrite(LM0, HIGH);
      digitalWrite(LM1, LOW);
      // Serial.println("left forward");
      break;
    case REV: //reverse
      digitalWrite(LM0, LOW);
      digitalWrite(LM1, HIGH);
      // Serial.println("left reverse");
      break;
    default:
      digitalWrite(LM0, LOW);
      digitalWrite(LM1, LOW);
      break;
  }

  switch(rightDir) {
    case STP: //stop
      digitalWrite(RM0, LOW);
      digitalWrite(RM1, LOW);
      // Serial.println("right stop");
      break;
    case FOR: //forward
      digitalWrite(RM0, HIGH);
      digitalWrite(RM1, LOW);
      // Serial.println("right forward");
      break;
    case REV: //reverse
      digitalWrite(RM0, LOW);
      digitalWrite(RM1, HIGH);
      // Serial.println("right reverse");
      break;
    default:
      digitalWrite(RM0, LOW);
      digitalWrite(RM1, LOW);
      break;
  }
}

// Create your own stunts with a combination of motor
// commands and delay statements
void stunt(int stuntId) {
	switch (stuntId) {
	    case 1:
	      // do stunt 1
	      break;
	    case 2:
	      motor(FOR,FOR);
	      delay(200);
	      motor(REV,REV);
	      delay(200);
	      motor(FOR,REV);
	      delay(500);
	      motor(REV,FOR);
	      delay(500);
	      motor(STP,STP);
	      break;
	    case 3:
	      // do stunt 3
	      break;
	    default:
	    	break;
	}
}

void setupWiFi()
{
  WiFi.mode(WIFI_AP);

  uint8_t mac[WL_MAC_ADDR_LENGTH];
  WiFi.softAPmacAddress(mac);
  
  String AP_NameString = "WifiStunt";

  char AP_NameChar[AP_NameString.length() + 1];
  memset(AP_NameChar, 0, AP_NameString.length() + 1);

  for (int i=0; i<AP_NameString.length(); i++)
    AP_NameChar[i] = AP_NameString.charAt(i);

  WiFi.softAP(AP_NameChar, WiFiAPPSK);
}

